import dataclasses
import datetime
from typing import List, Set


@dataclasses.dataclass()
class Answer: 
    letter: str
    text: str


@dataclasses.dataclass()
class Question: 
    designation: str
    correct_answer: str
    is_withdrawn: bool
    text: str = None
    figure_designation: str = None
    answers: List[Answer] = dataclasses.field(default_factory = list)


@dataclasses.dataclass()
class Group:
    designation: str
    description: str
    questions: List[Question] = dataclasses.field(default_factory = list)


@dataclasses.dataclass()
class SubElement:
    designation: str
    description: str
    expected_group_count: int
    question_groups: List[Group] = dataclasses.field(default_factory = list)

    def total_questions(self):
        nr = 0
        for g in self.question_groups:
            nr += len(g.questions)
        return nr


@dataclasses.dataclass()
class Element:
    name: str
    number: int
    version: int = 1
    date_created: datetime.date = None
    date_expires: datetime.date = None
    subelements: List[SubElement] = dataclasses.field(default_factory = list)
    withdrawn_questions: Set[str] = dataclasses.field(default_factory = set)
    graphics_directory: str = ''

    def total_questions(self):
        nr = 0
        for se in self.subelements:
            for g in se.question_groups:
                nr += len(g.questions)
        return nr
