package net.rten.palmve;

import androidx.test.ext.junit.runners.AndroidJUnit4;

import org.junit.Test;
import org.junit.runner.RunWith;

import java.time.format.DateTimeFormatter;

import static org.junit.Assert.assertEquals;

/**
 * Unit tests for the LocalPool class.
 */
@RunWith(AndroidJUnit4.class)
public class LocalPoolTests {
    private static class ExpectedDates {
        public final String dateField;
        public final String from;
        public final String to;

        public ExpectedDates(String df, String f, String t) {
            dateField = df;
            from = f;
            to = t;
        }
    }

    private static final ExpectedDates[] sExpectedDates;

    /**
     * Test that parsing date ranges works.
     */
    @Test
    public void testDateFieldParsing() {
        for (ExpectedDates dates : sExpectedDates) {
            LocalPool.EffectiveDateRange edr = LocalPool.getEffectiveDateRange(dates.dateField);
            assertEquals(dates.from, edr.from.toZonedDateTime().toLocalDate().format(DateTimeFormatter.ISO_DATE));
            assertEquals(dates.to, edr.to.toZonedDateTime().toLocalDate().format(DateTimeFormatter.ISO_DATE));
        }
    }

    static {
        // these are all date ranges from the real stuff
        sExpectedDates = new ExpectedDates[]{
                new ExpectedDates("July 1, 2008 through June 30, 2012",
                        "2008-07-01", "2012-06-30"),
                new ExpectedDates("July 1, 2011 through June 30, 2015",
                        "2011-07-01", "2015-06-30"),
                new ExpectedDates("July 1, 2007 through June 30, 2011",
                        "2007-07-01", "2011-06-30"),
                // ugh: note the mis-spelling
                new ExpectedDates("July 1, 2010 throuh June 30, 2014",
                        "2010-07-01", "2014-06-30")};
    }
}
