package net.rten.palmve;

import android.content.Context;

import androidx.test.core.app.ApplicationProvider;
import androidx.test.platform.app.InstrumentationRegistry;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
public class TestEnvironment {

    private final File _baseDir;
    private final File _scratchDir;

    @Inject
    public TestEnvironment() {
        Context appCtx = ApplicationProvider.getApplicationContext();
        _baseDir = new File(appCtx.getExternalFilesDir(null) + File.separator + "test_env");
        _scratchDir = new File(_baseDir + File.separator + "internal");
        try {
            setUp();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void setUp() throws IOException {
        deleteRecursive(_baseDir);
        if (!_scratchDir.mkdirs()) {
            throw new RuntimeException("failed to create dir " + _scratchDir.toString());
        }
        copyResource("test_pool.sqlite3", getPoolDbFile());
        copyResource("test_history.db", getHistoryDbFile());
    }

    public void reset() throws IOException {
        deleteRecursive(_scratchDir);
        if (!_scratchDir.mkdirs()) {
            throw new RuntimeException("failed to create dir " + _scratchDir.toString());
        }
        copyResource("test_history.db", getHistoryDbFile());
    }

    public File getScratchDir() {
        return _scratchDir;
    }

    public File getPoolDbFile() {
        return new File(_scratchDir + File.separator + "test_pool.sqlite3");
    }

    public File getHistoryDbFile() {
        return new File(_baseDir + File.separator + "test_history.db");
    }

    protected void copyResource(String assetName, File toLocation) throws IOException {
        Context testCtx = InstrumentationRegistry.getInstrumentation().getContext();
        InputStream inF = testCtx.getResources().getAssets().open(assetName);
        FileOutputStream outF = new FileOutputStream(toLocation);
        byte[] inputBuffer = new byte[1024];
        while (true) {
            int byteCount = inF.read(inputBuffer);
            if (byteCount <= 0) {
                break;
            }
            outF.write(inputBuffer, 0, byteCount);
        }
        outF.close();
    }

    private void deleteRecursive(File f) {
        if (f.isDirectory()) {
            File[] files = f.listFiles();
            if (files != null) {
                for (File child : files) {
                    deleteRecursive(child);
                }
            }
        }
        if (f.exists() && !f.delete()) {
            throw new RuntimeException("failed to delete " + f.toString());
        }
    }
}
