/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.activities;

import android.app.SearchManager;
import android.content.Intent;
import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.LocalPool;
import net.rten.palmve.R;
import net.rten.palmve.data.pool.PoolDatabase;
import net.rten.palmve.fragments.PoolSearchResultsFragment;
import net.rten.palmve.models.PoolSearchResultsModel;
import net.rten.palmve.repository.PoolRepo;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import dagger.hilt.android.AndroidEntryPoint;

/**
 * Display search results.
 */
@AndroidEntryPoint
public final class PoolSearchResultsActivity extends AppCompatActivity {
    @Inject
    protected PoolRepo _pools;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.generic_activity);
        setupModel();

        if (savedInstanceState == null) {
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
            transaction.replace(R.id.generic_fragment, new PoolSearchResultsFragment());
            transaction.commit();
        }
    }

    private void setupModel() {
        List<PoolDatabase> dbs = new ArrayList<>();
        List<Integer> ids = new ArrayList<>();
        for (LocalPool pool : _pools.getLocalPools()) {
            dbs.add(pool.getRoomDb());
            ids.add(pool.getLocalPoolId());
        }

        // Setting the search term doesn't work the way it should - it must be set before
        // we call .setUp() - the model is not smart enough to notice it changing.
        PoolSearchResultsModel model = new ViewModelProvider(this).get(PoolSearchResultsModel.class);
        final Intent intent = getIntent();
        if (Intent.ACTION_SEARCH.equals(intent.getAction())) {
            model.setSearchTerm(intent.getStringExtra(SearchManager.QUERY));
        }
        model.setUp(dbs, ids);
    }
}
