/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.activities;

import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.R;
import net.rten.palmve.fragments.TestHistoryListFragment;
import net.rten.palmve.models.TestHistoryListModel;
import net.rten.palmve.repository.TestHistoryRepo;

import javax.inject.Inject;

import dagger.hilt.android.AndroidEntryPoint;

/**
 * This is just a list of the tests you've taken.
 */
@AndroidEntryPoint
public class TestHistoryListActivity extends AppCompatActivity {
    public static final String INTENT_POOL_NAME = "net.rten.palmve.poolName";
    public static final String INTENT_POOL_DATE = "net.rten.palmve.poolDate";

    @Inject
    protected TestHistoryRepo _testHistory;

    private TestHistoryListModel _model;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.generic_activity);
        final String poolName = getIntent().getStringExtra(INTENT_POOL_NAME);
        final String poolDateField = getIntent().getStringExtra(INTENT_POOL_DATE);

        _model = new ViewModelProvider(this).get(TestHistoryListModel.class);
        _model.setUp(poolName, poolDateField, _testHistory.getDb());

        setTitle(getString(R.string.TestHistoryList_Title, poolName));

        if (savedInstanceState == null) {
            FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();
            transaction.replace(R.id.generic_fragment, new TestHistoryListFragment());
            transaction.commit();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.test_history_list_options, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.testHistoryItemsOptionsClearHistory) {
            _model.asyncDeleteTestHistory();
            return true;
        }
        return false;
    }
}
