package net.rten.palmve.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import net.rten.palmve.R;
import net.rten.palmve.data.pool.PoolItemWithDescriptionAndDesignation;
import net.rten.palmve.utilities.GenericListBinder;
import net.rten.palmve.utilities.PoolItemDiffCallback;

public class GenericListFragment<
        HV extends PoolItemWithDescriptionAndDesignation,
        IV extends PoolItemWithDescriptionAndDesignation> extends Fragment {
    private static final String TAG = "GenericListFragment";

    private GenericListBinder<HV, IV> _binder;
    private ItemAdapter _adapter;
    private final DiffUtil.ItemCallback<IV> _diffCallback = new PoolItemDiffCallback<>();

    static private class ItemViewHolder extends RecyclerView.ViewHolder {
        private final TextView _tag;
        private final TextView _description;

        public ItemViewHolder(View view) {
            super(view);
            _tag = view.findViewById(R.id.genericListItemTag);
            _description = view.findViewById(R.id.genericListItemDescription);
        }
    }

    private class ItemAdapter extends ListAdapter<IV, ItemViewHolder> {
        public ItemAdapter() {
            super(_diffCallback);
        }

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View itemView = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.generic_list_item, parent, false);
            return new ItemViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            IV item = getItem(position);
            holder._tag.setText(item.designation);
            holder._description.setText(item.description);
            holder.itemView.setOnClickListener(
                    clickedView -> _binder.onItemIdClicked(getItem(position).id));
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.recycler_view_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        RecyclerView recyclerView = view.findViewById(R.id.recyclerViewList);
        LinearLayoutManager layoutManager = new LinearLayoutManager(getActivity());
        recyclerView.setLayoutManager(layoutManager);
        _adapter = new ItemAdapter();
        recyclerView.setAdapter(_adapter);
        recyclerView.addItemDecoration(new DividerItemDecoration(
                recyclerView.getContext(), DividerItemDecoration.VERTICAL));
    }

    public void onViewModelReady(GenericListBinder<HV, IV> binder) {
        _binder = binder;
        _binder.getListFromModel().observe(getViewLifecycleOwner(), items -> _adapter.submitList(items));
    }
}
