package net.rten.palmve.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.R;
import net.rten.palmve.activities.GenericListActivity;
import net.rten.palmve.activities.ViewFigureActivity;
import net.rten.palmve.data.pool.QuestionView;
import net.rten.palmve.models.QuestionModel;

public class QuestionFragment extends Fragment {
    private static final String TAG = "QuestionFragment";

    private QuestionModel _model;
    private TextView _questionId;
    private TextView _questionText;
    private TextView _withdrawn;
    private TextView _correctAnswer;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.question_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _questionId = view.findViewById(R.id.questionFragmentTag);
        _questionText = view.findViewById(R.id.questionFragmentText);
        _withdrawn = view.findViewById(R.id.questionFragmentWithdrawnValue);
        _correctAnswer = view.findViewById(R.id.questionFragmentAnswerValue);
        final TextView withdrawnLabel = view.findViewById(R.id.questionFragmentWithdrawnLabel);
        final Button viewFigure = view.findViewById(R.id.questionFragmentViewFigure);

        _model = new ViewModelProvider(requireActivity()).get(QuestionModel.class);
        _model.getHeaderDataCopy().observe(getViewLifecycleOwner(), question -> {
            if (question != null) {
                _questionId.setText(question.designation);
                _questionText.setText(question.description);
                _withdrawn.setText(question.withdrawn ? R.string.Standard_Yes : R.string.Standard_No);

                if (_model.getMode() == QuestionModel.Mode.answeringQuestion) {
                    view.findViewById(R.id.questionFragmentAnswer).setVisibility(View.GONE);
                } else {
                    _model.getHideAnswerOnBrowse().observe(getViewLifecycleOwner(), hide -> {
                        QuestionView q = _model.getHeaderDataCopy().getValue();
                        if (q != null) {
                            showOrHideAnswer(q, hide);
                        }
                    });
                }

                if (question.figureId == null) {
                    viewFigure.setVisibility(View.GONE);
                } else {
                    viewFigure.setVisibility(View.VISIBLE);
                    viewFigure.setOnClickListener(clicked -> {
                        Intent intent = new Intent(requireContext(), ViewFigureActivity.class);
                        intent.putExtra(GenericListActivity.INTENT_POOL_ID, _model.getLocalPoolId());
                        intent.putExtra(ViewFigureActivity.INTENT_FIGURE_ID,
                                question.figureId.longValue());
                        startActivity(intent);
                    });
                }

                if (_model.getHideWithdrawn()) {
                    withdrawnLabel.setVisibility(View.GONE);
                    _withdrawn.setVisibility(View.GONE);
                }
            }
        });
        if (_model.getMode() == QuestionModel.Mode.answeringQuestion) {
            _model.getHideAnswerOnBrowse().observe(getViewLifecycleOwner(), hide -> {
                QuestionView question = _model.getHeaderDataCopy().getValue();
                if (question != null) {
                    showOrHideAnswer(question, hide);
                }
            });
        }
    }

    private void showOrHideAnswer(QuestionView question, boolean hide) {
        if (hide) {
            _correctAnswer.setText(R.string.Browse_Answers_MaskingAnswer);
        } else {
            _correctAnswer.setText(AnswerFragment.getAnswerLetterResource(question.rightAnswer));
        }
    }
}
