package net.rten.palmve.fragments;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import net.rten.palmve.R;
import net.rten.palmve.activities.TestResultsActivity;
import net.rten.palmve.data.test.TestHistory;
import net.rten.palmve.data.test.TestState;
import net.rten.palmve.models.TestHistoryListModel;

import java.text.DateFormat;

public class TestHistoryListFragment extends Fragment {
    private static final String TAG = "TestHistoryListFragment";
    private final Diff _diff = new Diff();
    private final ItemAdapter _adapter = new ItemAdapter();

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.recycler_view_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        LinearLayoutManager layoutManager = new LinearLayoutManager(requireActivity());
        RecyclerView list = view.findViewById(R.id.recyclerViewList);
        list.setLayoutManager(layoutManager);
        list.setAdapter(_adapter);
        list.addItemDecoration(new DividerItemDecoration(
                list.getContext(), DividerItemDecoration.VERTICAL));

        TestHistoryListModel model = new ViewModelProvider(requireActivity()).get(TestHistoryListModel.class);
        model.getTestList().observe(getViewLifecycleOwner(), _adapter::submitList);
    }

    protected void onItemIdClicked(TestHistory test) {
        Intent intent = new Intent(getActivity(), TestResultsActivity.class);
        intent.putExtra(TestResultsActivity.INTENT_POOL_NAME, test.elementName);
        intent.putExtra(TestResultsActivity.INTENT_POOL_DATE, test.elementDateField);
        intent.putExtra(TestResultsActivity.INTENT_TEST_ID, test.id);
        startActivity(intent);
    }

    static private class ItemViewHolder extends RecyclerView.ViewHolder {
        private final ImageView _icon;
        private final TextView _tag, _description;

        public ItemViewHolder(View view) {
            super(view);
            _icon = view.findViewById(R.id.testHistoryListItemIcon);
            _tag = view.findViewById(R.id.testHistoryListItemTag);
            _description = view.findViewById(R.id.testHistoryListItemDescription);
        }
    }

    static private class Diff extends DiffUtil.ItemCallback<TestHistory> {
        @Override
        public boolean areItemsTheSame(@NonNull TestHistory oldItem, @NonNull TestHistory newItem) {
            return oldItem.id.equals(newItem.id);
        }

        @Override
        public boolean areContentsTheSame(@NonNull TestHistory oldItem, @NonNull TestHistory newItem) {
            return true;  // Individual items do not change.
        }
    }

    private class ItemAdapter extends ListAdapter<TestHistory, ItemViewHolder> {
        protected ItemAdapter() {
            super(_diff);
        }

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View itemView = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.test_history_list_item, parent, false);
            return new ItemViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            final TestHistory item = getItem(position);
            final boolean passed = (item.testState == TestState.PASSED);
            final int prefixStringResource = passed ? R.string.Standard_Passed : R.string.Standard_Failed;
            final Context context = holder.itemView.getContext();

            holder._icon.setImageResource(passed
                    ? R.drawable.ic_baseline_check_circle_outline_green_24
                    : R.drawable.ic_outline_cancel_red_24);
            holder._icon.setContentDescription(context.getString(prefixStringResource));
            holder._tag.setText(context.getString(R.string.TestHistoryStats_Percentage,
                    context.getString(prefixStringResource), item.testScorePercent));

            DateFormat dateFormat = DateFormat.getDateTimeInstance();
            holder._description.setText(dateFormat.format(item.dateCompleted.getTime()));

            holder.itemView.setOnClickListener(
                    clickedView -> onItemIdClicked(getItem(position)));
        }
    }
}
