package net.rten.palmve.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import net.rten.palmve.R;
import net.rten.palmve.data.test.StatsDao;
import net.rten.palmve.models.TestHistoryStatsModel;

import java.util.Locale;

public class TestHistoryStatsConfigFragment extends Fragment {
    private static final String TAG = "TestHistoryStatsCFragment";
    private TestHistoryStatsModel _model;
    private TextView _testsTakenText;
    private TextView _testsPassedText;
    private TextView _passingPctText;
    private Spinner _timeSpanSpinner;
    private Spinner _detailSpinner;
    private Spinner _limitSpinner;
    private final AdapterView.OnItemSelectedListener _resetModelParameters = new AdapterView.OnItemSelectedListener() {
        @Override
        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
            onResetModelParameters();
        }

        @Override
        public void onNothingSelected(AdapterView<?> parent) {
        }
    };

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.test_history_stats_config_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _model = new ViewModelProvider(requireActivity()).get(TestHistoryStatsModel.class);

        _testsTakenText = view.findViewById(R.id.testHistoryStatsTestsTaken);
        _testsPassedText = view.findViewById(R.id.testHistoryStatsTestsPassed);
        _passingPctText = view.findViewById(R.id.testHistoryStatsPassingPercentage);

        // put the time frames in the array adapter
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(
                requireActivity(), R.array.testHistoryStatsTimeframeEntries,
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        _timeSpanSpinner = view.findViewById(R.id.testHistoryStatsTimespan);
        _timeSpanSpinner.setAdapter(adapter);
        _timeSpanSpinner.setOnItemSelectedListener(_resetModelParameters);

        // and do the same for detail types
        adapter = ArrayAdapter.createFromResource(requireActivity(),
                R.array.testHistoryStatsDetailsEntries,
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        _detailSpinner = view.findViewById(R.id.testHistoryStatsDetailShow);
        _detailSpinner.setAdapter(adapter);
        _detailSpinner.setOnItemSelectedListener(_resetModelParameters);

        // limits, too
        adapter = ArrayAdapter.createFromResource(requireActivity(),
                R.array.testHistoryStatsLimitEntries,
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        _limitSpinner = view.findViewById(R.id.testHistoryStatsDetailLimit);
        _limitSpinner.setAdapter(adapter);
        _limitSpinner.setOnItemSelectedListener(_resetModelParameters);

        setUiFromReportParams();

        // bind the count info into the various header UI elements
        _model.getCounts().observe(getViewLifecycleOwner(), stats -> {
            _testsTakenText.setText(String.format(Locale.getDefault(), "%1$d", _model.getTestCount()));
            _testsPassedText.setText(String.format(Locale.getDefault(), "%1$d", _model.getPassingTestCount()));
            if (_model.getTestCount() != 0) {
                _passingPctText.setText(String.format(Locale.getDefault(), "%1$d%%",
                        _model.getPassingTestCount() * 100 / _model.getTestCount()));
            } else {
                _passingPctText.setText(R.string.TestHistoryStats_NoTests);
            }
        });
    }

    /**
     * Update the UI spinners to reflect the params in the model.
     */
    private void setUiFromReportParams() {
        // It would be better if these indices were not hard coded like this, or if I just threw
        // out the stupid arrays in the resource files.
        final StatsDao.DetailReportType reportType = _model.getReportType();
        final boolean forCorrectAnswers = _model.forCorrectAnswers();
        if (reportType == StatsDao.DetailReportType.questionGroups && forCorrectAnswers) {
            _detailSpinner.setSelection(3);
        } else if (reportType == StatsDao.DetailReportType.questionGroups) {
            _detailSpinner.setSelection(1);
        } else if (reportType == StatsDao.DetailReportType.questions && forCorrectAnswers) {
            _detailSpinner.setSelection(2);
        } else if (reportType == StatsDao.DetailReportType.questions) {
            _detailSpinner.setSelection(0);
        }

        final int limit = _model.getLimit();
        final int[] limits = getResources().getIntArray(R.array.testHistoryStatsLimitValues);
        for (int i = 0; i < limits.length; ++i) {
            if (limits[i] == limit) {
                _limitSpinner.setSelection(i);
                break;
            }
        }

        final int secondsAgo = _model.getSecondsAgo();
        final int[] spans = getResources().getIntArray(R.array.testHistoryStatsTimeframeValues);
        for (int i = 0; i < spans.length; ++i) {
            if (spans[i] == secondsAgo) {
                _timeSpanSpinner.setSelection(i);
                break;
            }
        }
    }

    /**
     * Update the model to reflect the params in the UI.
     */
    private void onResetModelParameters() {
        final String typeStr = getResources().getStringArray(R.array.testHistoryStatsDetailsValues)[
                _detailSpinner.getSelectedItemPosition()];
        final int limit = getResources().getIntArray(R.array.testHistoryStatsLimitValues)[
                _limitSpinner.getSelectedItemPosition()];
        final int timeSpan = getResources().getIntArray(R.array.testHistoryStatsTimeframeValues)[
                _timeSpanSpinner.getSelectedItemPosition()];

        StatsDao.DetailReportType reportType;
        boolean correctAnswers;
        switch (typeStr) {
            case "statDetailMissedQs":
                reportType = StatsDao.DetailReportType.questions;
                correctAnswers = false;
                break;
            case "statDetailMissedQGroups":
                reportType = StatsDao.DetailReportType.questionGroups;
                correctAnswers = false;
                break;
            case "statDetailGoodQs":
                reportType = StatsDao.DetailReportType.questions;
                correctAnswers = true;
                break;
            case "statDetailGoodQGroups":
            default:
                reportType = StatsDao.DetailReportType.questionGroups;
                correctAnswers = true;
                break;
        }

        _model.setReportParams(reportType, correctAnswers, timeSpan, limit);
    }
}
