package net.rten.palmve.fragments;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import net.rten.palmve.R;
import net.rten.palmve.activities.GenericListActivity;
import net.rten.palmve.activities.ViewQuestionAndAnswersActivity;
import net.rten.palmve.activities.QuestionGroupActivity;
import net.rten.palmve.models.TestHistoryStatsModel;

public class TestHistoryStatsListFragment extends Fragment {
    private static final String TAG = "TestHistoryStatsLFragment";
    private final DiffUtil.ItemCallback<TestHistoryStatsModel.Stats> _diffCallback = new DiffUtil.ItemCallback<TestHistoryStatsModel.Stats>() {
        @Override
        public boolean areItemsTheSame(@NonNull TestHistoryStatsModel.Stats oldItem, @NonNull TestHistoryStatsModel.Stats newItem) {
            return oldItem.questionOrGroupDescription.equals(newItem.questionOrGroupDescription);
        }

        @Override
        public boolean areContentsTheSame(@NonNull TestHistoryStatsModel.Stats oldItem, @NonNull TestHistoryStatsModel.Stats newItem) {
            return oldItem.questionOrGroupDescription.equals(newItem.questionOrGroupDescription);
        }
    };
    private final ItemAdapter _adapter = new ItemAdapter();
    private TestHistoryStatsModel _model;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.test_history_stats_list_fragment, container, false);
        rootView.setTag(TAG);
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        _model = new ViewModelProvider(requireActivity()).get(TestHistoryStatsModel.class);

        // setup the recycler view
        LinearLayoutManager layoutManager = new LinearLayoutManager(requireActivity());
        RecyclerView list = view.findViewById(R.id.testHistoryStatsListFragmentRecyclerView);
        list.setLayoutManager(layoutManager);
        list.setAdapter(_adapter);
        list.addItemDecoration(new DividerItemDecoration(
                list.getContext(), DividerItemDecoration.VERTICAL));

        // the first observe() seeds the list of things, the second is required to load the
        // question or question group strings, as these come after the initial list is ready.
        _model.getStats().observe(getViewLifecycleOwner(), _adapter::submitList);
        _model.getDescriptions().observe(getViewLifecycleOwner(), _adapter::notifyItemChanged);
    }

    private void onItemIdClicked(TestHistoryStatsModel.Stats item) {
        Intent intent = null;
        if (item.testStats.questionId != null) {
            intent = new Intent(requireContext(), ViewQuestionAndAnswersActivity.class);
            intent.putExtra(ViewQuestionAndAnswersActivity.INTENT_QUESTION_ID,
                    item.testStats.questionId);
        } else if (item.testStats.questionGroupId != null) {
            intent = new Intent(requireContext(), QuestionGroupActivity.class);
            intent.putExtra(QuestionGroupActivity.INTENT_QUESTION_GROUP_ID,
                    item.testStats.questionGroupId);
        }

        if (intent != null) {
            // TODO: it's annoying we have to pass INTENT_POOL_ID like this, along with the strings.
            intent.putExtra(GenericListActivity.INTENT_POOL_ID,
                    requireActivity().getIntent().getIntExtra(GenericListActivity.INTENT_POOL_ID, 0));
            startActivity(intent);
        }
    }

    static private class ItemViewHolder extends RecyclerView.ViewHolder {
        private final TextView _title, _description;

        public ItemViewHolder(View view) {
            super(view);
            _title = view.findViewById(R.id.testHistoryStatsItemTag);
            _description = view.findViewById(R.id.testHistoryStatsItemDescription);
        }
    }

    private class ItemAdapter extends ListAdapter<TestHistoryStatsModel.Stats, ItemViewHolder> {
        protected ItemAdapter() {
            super(_diffCallback);
        }

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View itemView = LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.test_history_stats_item, parent, false);
            return new ItemViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ItemViewHolder holder, int position) {
            final int tagId = _model.forCorrectAnswers()
                    ? R.string.Standard_Correct
                    : R.string.Standard_Incorrect;
            final TestHistoryStatsModel.Stats item = getItem(position);
            holder._title.setText(
                    holder._title.getContext().getString(
                            R.string.TestHistoryStats_GroupSummary,
                            item.testStats.questionCount,
                            holder._title.getContext().getString(tagId),
                            item.testStats.designation));
            holder._description.setText(item.questionOrGroupDescription);
            holder.itemView.setOnClickListener(
                    clickedView -> onItemIdClicked(getItem(position)));
        }
    }
}
