package net.rten.palmve.models;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import net.rten.palmve.Preferences;
import net.rten.palmve.data.pool.Answer;
import net.rten.palmve.data.pool.AnswerView;
import net.rten.palmve.data.pool.PoolDatabase;
import net.rten.palmve.data.pool.QuestionView;

import java.util.List;

public class QuestionModel extends GenericListModel<QuestionView, AnswerView> {
    private final MutableLiveData<Answer.AnswerCode> _firstAnswerSelected = new MutableLiveData<>();
    private final MutableLiveData<Answer.AnswerCode> _answerSelected = new MutableLiveData<>();
    private final MutableLiveData<Boolean> _questionBrowserMaskAnswers = new MutableLiveData<>(false);
    private int _localPoolId;
    private boolean _questionBrowserHideWithdrawn;
    private Mode _mode = Mode.browsingQuestion;
    private Preferences.WrongAnswerBehavior _testTakingWrongAnswer;

    public Mode getMode() {
        return _mode;
    }

    public void setMode(Mode mode) {
        _mode = mode;
    }

    public LiveData<Answer.AnswerCode> getSelectedAnswer() {
        return _answerSelected;
    }

    @Override
    public void setPrimaryId(long id) {
        // Clear out any active answer state before we load the next question.
        _firstAnswerSelected.setValue(null);
        _answerSelected.setValue(null);
        super.setPrimaryId(id);
    }

    public void setSelectedAnswer(Answer.AnswerCode to) {
        _answerSelected.setValue(to);
        if (_firstAnswerSelected.getValue() == null) {
            _firstAnswerSelected.setValue(to);
        }
    }

    public LiveData<Answer.AnswerCode> getFirstAnswerSelected() {
        return _firstAnswerSelected;
    }

    public void setPoolId(int localPoolId) {
        _localPoolId = localPoolId;
    }

    public int getLocalPoolId() {
        return _localPoolId;
    }

    public Preferences.WrongAnswerBehavior getWrongAnswerBehavior() {
        return _testTakingWrongAnswer;
    }

    public void setWrongAnswerBehavior(Preferences.WrongAnswerBehavior testTakingWrongAnswer) {
        _testTakingWrongAnswer = testTakingWrongAnswer;
    }

    public boolean getHideWithdrawn() {
        return _questionBrowserHideWithdrawn || _mode == Mode.answeringQuestion;
    }

    public void setHideWithdrawn(boolean questionBrowserHideWithdrawn) {
        _questionBrowserHideWithdrawn = questionBrowserHideWithdrawn;
    }

    public LiveData<Boolean> getHideAnswerOnBrowse() {
        return _questionBrowserMaskAnswers;
    }

    public void setHideAnswerOnBrowse(boolean questionBrowserMaskAnswers) {
        _questionBrowserMaskAnswers.setValue(questionBrowserMaskAnswers);
    }

    @Override
    protected LiveData<QuestionView> getHeaderDataFromDao(PoolDatabase poolDb) {
        return poolDb.questionDao().getQuestionByIdLiveData(getPrimaryId());
    }

    @Override
    protected LiveData<List<AnswerView>> getItemDataFromDao(PoolDatabase poolDb) {
        return poolDb.questionDao().getAnswersByQuestionIdLiveData(getPrimaryId());
    }

    public enum Mode {
        answeringQuestion, browsingQuestion
    }
}
