/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.utilities;

import androidx.annotation.NonNull;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Reads from an input stream, and copies read bytes to a local output
 * stream. This class assumes ownership of the output stream and will close
 * it when you close the input stream.
 */
public class FileCopierInputStream extends FilterInputStream {
    private OutputStream _outputStream;

    public FileCopierInputStream(InputStream in, OutputStream copyTo) {
        super(in);
        _outputStream = copyTo;
    }

    @Override
    public boolean markSupported() {
        return false;
    }

    @Override
    public void mark(int readLimit) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void reset() {
        throw new UnsupportedOperationException();
    }

    @Override
    public void close() throws IOException {
        in.close();
        if (_outputStream != null) {
            _outputStream.close();
        }
    }

    @Override
    public int available() throws IOException {
        return in.available();
    }

    @Override
    public long skip(long n) {
        throw new UnsupportedOperationException();
    }

    @Override
    public int read() throws IOException {
        int bytesRead = in.read();
        if (bytesRead >= 0) {
            _outputStream.write(bytesRead);
        }
        return bytesRead;
    }

    @Override
    public int read(@NonNull byte[] b, int off, int len) throws IOException {
        int bytesRead = in.read(b, off, len);
        if (bytesRead < 0) {
            _outputStream.close();
            _outputStream = null;
        } else {
            _outputStream.write(b, off, bytesRead);
        }
        return bytesRead;
    }
}
