/*
 * PalmVE - Android amateur radio test practice program
 * Copyright (C) 2010 Michael A Ballbach
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.rten.palmve.utilities;

import net.rten.palmve.RemotePool;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;

import java.util.ArrayList;

/**
 * Xml parser for the remote pool XML list.
 */
public class RemotePoolXmlParser implements ContentHandler {
    private ArrayList<RemotePool> _remotePools;
    private RemotePool _currentPool;

    public ArrayList<RemotePool> getRemotePools() {
        return _remotePools;
    }

    // ContentHandler: Methods we Need
    public void startDocument() {
        _remotePools = new ArrayList<>();
    }

    public void startElement(String uri, String localName, String qName,
                             Attributes attributes) {
        if (localName.equals("pool")) {
            // Size was not always included in the pool directory, so we default to zero.
            long size = 0;
            String sizeString = attributes.getValue("size");
            if (sizeString != null) {
                size = Long.parseLong(sizeString);
            }

            _currentPool = new RemotePool(Integer.parseInt(
                    attributes.getValue("element")),
                    Integer.parseInt(attributes.getValue("format")),
                    Integer.parseInt(attributes.getValue("version")),
                    attributes.getValue("name"), attributes.getValue("datefield"),
                    attributes.getValue("sum"), size);
        }
    }

    public void characters(char[] ch, int start, int length) {
        if (_currentPool != null) {
            _currentPool.setRemoteFilename(new String(ch, start, length));
        }
    }

    public void endElement(String uri, String localName, String qName) {
        if (_currentPool != null && localName.equals("pool")) {
            _remotePools.add(_currentPool);
            _currentPool = null;
        }
    }

    // ContentHandler: Unused Methods
    public void endDocument() {
    }

    public void startPrefixMapping(String prefix, String uri) {
    }

    public void endPrefixMapping(String prefix) {
    }

    public void ignorableWhitespace(char[] ch, int start, int length) {
    }

    public void processingInstruction(String target, String data) {
    }

    public void setDocumentLocator(Locator locator) {
    }

    public void skippedEntity(String name) {
    }
}
